const { resolve } = require('path');
const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const webpack = require('webpack');
const WorkboxPlugin = require('workbox-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const OptimizeCssAssetsWebpackPlugin = require('optimize-css-assets-webpack-plugin');
const PrerenderSPAPlugin = require('prerender-spa-plugin');
const PreloadWebpackPlugin = require('preload-webpack-plugin');

const merge = require('webpack-merge');

const base = require('./webpack.base.js');
const env = require(`../config/${process.env.ENV_MARK}.env`);
console.log('env', env.NODE_ENV);
const os = require('os');
function resolveRoot(dir) {
  return path.join(__dirname, '..', dir);
}
module.exports = merge(base, {
  devtool: 'source-map',
  output: {
    publicPath: '/new/',
    filename: 'js/[name].[contenthash:8].js'
  },
  module: {
    rules: [
      {
        enforce: 'pre',
        test: /\.js$/,
        exclude: /node_modules/,
        include: resolve(__dirname, '/src/js'),
        loader: 'eslint-loader'
      },
      {
        oneOf: [
          {
            test: /\.(html)$/,
            loader: 'html-loader'
          },
          {
            test: /\.(js|jsx)$/,
            use: [
              {
                loader: 'thread-loader',
                options: {
                  workers: os.cpus().length
                }
              },
              {
                loader: 'babel-loader',
                options: {
                  presets: [
                    '@babel/preset-react',
                    //tree shaking 按需加载babel-polifill
                    ['@babel/preset-env', { modules: false, useBuiltIns: 'false', corejs: 2 }]
                  ],
                  plugins: [
                    //支持
                    '@babel/plugin-syntax-dynamic-import',
                    //true是less， 可以写'css' 如果不用less
                    // ['import', { libraryName: 'antd-mobile', style: true }],
                    ['@babel/plugin-proposal-class-properties', { loose: true }]
                  ],
                  cacheDirectory: true
                }
              }
            ]
          },
          {
            test: /\.(css)$/,
            use: [
              MiniCssExtractPlugin.loader,
              {
                loader: 'css-loader',
                options: {
                  modules: false,
                  localIdentName: 'css/[local]--[hash:base64:5]'
                }
              }
            ]
          },
          {
            test: /\.(less)$/,
            use: [
              MiniCssExtractPlugin.loader,
              {
                loader: 'css-loader',
                options: {
                  modules: false,
                  localIdentName: 'css/[local]--[hash:base64:5]'
                }
              },
              { loader: 'postcss-loader' },
              {
                loader: 'less-loader',
                options: { javascriptEnabled: true }
              }
            ]
          },
          {
            test: /\.(jpg|jpeg|bmp|svg|png|webp|gif)$/,
            use: [
              {
                loader: 'url-loader',
                options: {
                  limit: 8 * 1024,
                  name: '[name].[hash:8].[ext]',
                  outputPath: '/img'
                }
              },
              {
                loader: 'img-loader',
                options: {
                  plugins: [
                    require('imagemin-gifsicle')({
                      interlaced: false
                    }),
                    require('imagemin-mozjpeg')({
                      progressive: true,
                      arithmetic: false
                    }),
                    require('imagemin-pngquant')({
                      floyd: 0.5,
                      speed: 2
                    }),
                    require('imagemin-svgo')({
                      plugins: [{ removeTitle: true }, { convertPathData: false }]
                    })
                  ]
                }
              }
            ]
          },
          {
            exclude: /\.(js|json|less|css|jsx)$/,
            loader: 'file-loader',
            options: {
              outputPath: 'media/',
              name: '[name].[contenthash:8].[ext]'
            }
          }
        ]
      }
    ]
  },
  plugins: [
    new webpack.DefinePlugin({
      'process.env': JSON.stringify(env)
    }),
    new HtmlWebpackPlugin({
      template: resolveRoot('index.html'),
      favicon: 'favicon.ico',
      minify: {
        removeComments: true,
        collapseWhitespace: true,
        removeRedundantAttributes: true,
        useShortDoctype: true,
        removeEmptyAttributes: true,
        removeStyleLinkTypeAttributes: true,
        keepClosingSlash: true,
        minifyJS: true,
        minifyCSS: true,
        minifyURLs: true
      }
    }),
    new MiniCssExtractPlugin({
      filename: 'css/[name].[contenthash:8].css'
    }),
    new CleanWebpackPlugin(),
    new OptimizeCssAssetsWebpackPlugin({
      cssProcessPluginOptions: {
        preset: ['default', { discardComments: { removeAll: true } }]
      }
    }),
    new webpack.HashedModuleIdsPlugin()
    // new PrerenderSPAPlugin({
    //   routes: ['/', '/home', '/shop'],
    //   staticDir: resolve(__dirname, '../dist')
    // })

    // new PreloadWebpackPlugin({
    //   rel: 'preload',
    //   as(entry) {
    //     if (/\.css$/.test(entry)) return 'style';
    //     if (/\.woff$/.test(entry)) return 'font';
    //     if (/\.png$/.test(entry)) return 'image';
    //     return 'script';
    //   },
    //   include: 'allChunks'
    //   //include: ['app']
    // }),
    // new WorkboxPlugin.GenerateSW({
    //   clientsClaim: true,
    //   skipWaiting: true,
    //   importWorkboxFrom: 'local',
    //   include: [/\.js$/, /\.css$/, /\.html$/, /\.jpg/, /\.jpeg/, /\.svg/, /\.webp/, /\.png/]
    // }),
  ],
  mode: 'production',
  optimization: {
    runtimeChunk: true,
    splitChunks: {
      chunks: 'async',
      minSize: 30000, // 形成一个新代码块最小的体积
      // maxAsyncRequests: 5, // 按需加载时候最大的并行请求数
      // maxInitialRequests: 3, // 最大初始化请求数
      automaticNameDelimiter: '~', // 打包分割符
      name: true,
      cacheGroups: {
        vendor: {
          // 打包两个页面的公共代码
          test: /react|react-dom|react-router-dom|babel-polyfill|antd-mobile/,
          chunks: 'all',
          priority: 5 // 优先
        },
        moment: {
          test: /moment/,
          chunks: 'all',
          name: 'moment', // 单独将 elementUI 拆包
          priority: 20 // 权重需大于其它缓存组
        },
        // 'async-commons': {
        //   // 异步加载公共包、组件等
        //   chunks: 'async',
        //   minChunks: 2,
        //   name: 'async-commons',
        //   priority: 10
        // },
        commons: {
          // 其他同步加载公共包
          chunks: 'all',
          priority: 2,
          minChunks: 2
        }
      }
    }
  }
});
